@@ -39,6 +39,9 @@ public class UIText extends NLS {
 	/** */
 	public static String AbstractRebaseCommand_DialogTitle;
 
+	/** */
+	public static String AbstractReflogCommandHandler_NoInput;
+
 	/** */
 	public static String Activator_DefaultRepoFolderIsFile;
 
@@ -207,6 +210,9 @@ public class UIText extends NLS {
 	/** */
 	public static String EGitCredentialsProvider_errorReadingCredentials;
 
+	/** */
+	public static String EGitCredentialsProvider_FailedToClearCredentials;
+
 	/** */
 	public static String EgitUiUtils_CouldNotOpenEditorMessage;
 
@@ -303,6 +309,9 @@ public class UIText extends NLS {
 	/** */
 	public static String GitCloneWizard_title;
 
+	/** */
+	public static String GitCloneWizard_jobImportProjects;
+
 	/** */
 	public static String GitCloneWizard_jobName;
 
@@ -312,6 +321,12 @@ public class UIText extends NLS {
 	/** */
 	public static String GitCloneWizard_errorCannotCreate;
 
+	/** */
+	public static String GitCloneWizard_MissingNotesMessage;
+
+	/** */
+	public static String GitCloneWizard_MissingNotesTitle;
+
 	/** */
 	public static String GitDecoratorPreferencePage_bindingRepositoryNameFlag;
 
@@ -501,6 +516,9 @@ public class UIText extends NLS {
 	/** */
 	public static String GitHistoryPage_ShowAllBranchesMenuLabel;
 
+	/** */
+	public static String GitHistoryPage_FollowRenames;
+
 	/** */
 	public static String GitHistoryPage_FilterSubMenuLabel;
 
@@ -570,6 +588,9 @@ public class UIText extends NLS {
 	/** */
 	public static String GitPreferenceRoot_SynchronizeView;
 
+	/** */
+	public static String ProjectsPreferencePage_AutoShareProjects;
+
 	/** */
 	public static String GitProjectPropertyPage_LabelBranch;
 
@@ -654,6 +675,18 @@ public class UIText extends NLS {
 	/** */
 	public static String RebaseResultDialog_NextStepsDoNothing;
 
+	/** */
+	public static String RebaseResultDialog_notInWorkspace;
+
+	/** */
+	public static String RebaseResultDialog_notInWorkspaceMessage;
+
+	/** */
+	public static String RebaseResultDialog_notShared;
+
+	/** */
+	public static String RebaseResultDialog_notSharedMessage;
+
 	/** */
 	public static String RebaseResultDialog_SkipCommitButton;
 
@@ -840,6 +873,12 @@ public class UIText extends NLS {
 	/** */
 	public static String CherryPickHandler_JobName;
 
+	/** */
+	public static String CherryPickHandler_ConfirmMessage;
+
+	/** */
+	public static String CherryPickHandler_ConfirmTitle;
+
 	/** */
 	public static String CherryPickHandler_unknown;
 
@@ -861,6 +900,9 @@ public class UIText extends NLS {
 	/** */
 	public static String CompareTargetSelectionDialog_CompareTitle;
 
+	/** */
+	public static String CompareTargetSelectionDialog_CompareTitleEmptyPath;
+
 	/** */
 	public static String CompareTargetSelectionDialog_WindowTitle;
 
@@ -1098,6 +1140,9 @@ public class UIText extends NLS {
 	/** */
 	public static String CloneDestinationPage_groupConfiguration;
 
+	/** */
+	public static String CloneDestinationPage_groupProjects;
+
 	/** */
 	public static String CloneDestinationPage_promptDirectory;
 
@@ -1125,6 +1170,9 @@ public class UIText extends NLS {
 	/** */
 	public static String CloneDestinationPage_errorRemoteNameRequired;
 
+	/** */
+	public static String CloneDestinationPage_importButton;
+
 	/** */
 	public static String RefContentProposal_blob;
 
@@ -1153,16 +1201,19 @@ public class UIText extends NLS {
 	public static String RefContentProposal_unknownObject;
 
 	/** */
-	public static String ReflogView_ErrorOnOpenCommit;
+	public static String ReflogView_DateColumnHeader;
 
 	/** */
-	public static String ReflogView_FromColumnHeader;
+	public static String ReflogView_ErrorOnOpenCommit;
 
 	/** */
 	public static String ReflogView_MessageColumnHeader;
 
 	/** */
-	public static String ReflogView_ToColumnHeader;
+	public static String ReflogView_CommitColumnHeader;
+
+	/** */
+	public static String ReflogView_CommitMessageColumnHeader;
 
 	/** */
 	public static String RefSelectionDialog_Messsage;
@@ -1893,6 +1944,12 @@ public class UIText extends NLS {
 	/** */
 	public static String GlobalConfigurationPreferencePage_repositorySettingNoRepositories;
 
+	/** */
+	public static String SpellcheckableMessageArea_redo;
+
+	/** */
+	public static String SpellcheckableMessageArea_undo;
+
 	/** */
 	public static String SpellCheckingMessageArea_copy;
 
@@ -2631,6 +2688,12 @@ public class UIText extends NLS {
 	/** */
 	public static String DeleteBranchDialog_WindowTitle;
 
+	/** */
+	public static String DeleteBranchOnCommitHandler_SelectBranchDialogMessage;
+
+	/** */
+	public static String DeleteBranchOnCommitHandler_SelectBranchDialogTitle;
+
 	/** */
 	public static String DeleteRepositoryConfirmDialog_DeleteRepositoryMessage;
 
@@ -2643,6 +2706,18 @@ public class UIText extends NLS {
 	/** */
 	public static String DeleteRepositoryConfirmDialog_DeleteWorkingDirectoryCheckbox;
 
+	/** */
+	public static String DeleteTagCommand_messageConfirmMultipleTag;
+
+	/** */
+	public static String DeleteTagCommand_messageConfirmSingleTag;
+
+	/** */
+	public static String DeleteTagCommand_taskName;
+
+	/** */
+	public static String DeleteTagCommand_titleConfirm;
+
 	/** */
 	public static String IgnoreActionHandler_addToGitignore;
 
@@ -2658,18 +2733,12 @@ public class UIText extends NLS {
 	/** */
 	public static String RepositoriesView_ClipboardContentNotDirectoryOrURIMessage;
 
-	/** */
-	public static String RepositoriesView_ConfirmBranchDeletionMessage;
-
 	/** */
 	public static String RepositoriesView_ConfirmDeleteRemoteHeader;
 
 	/** */
 	public static String RepositoriesView_ConfirmDeleteRemoteMessage;
 
-	/** */
-	public static String RepositoriesView_ConfirmDeleteTitle;
-
 	/** */
 	public static String RepositoriesView_ConfirmProjectDeletion_Question;
 
@@ -2688,6 +2757,18 @@ public class UIText extends NLS {
 	/** */
 	public static String RepositoriesView_ExceptionLookingUpRepoMessage;
 
+	/** */
+	public static String RepositoriesView_linkAdd;
+
+	/** */
+	public static String RepositoriesView_linkClone;
+
+	/** */
+	public static String RepositoriesView_linkCreate;
+
+	/** */
+	public static String RepositoriesView_messsageEmpty;
+
 	/** */
 	public static String RepositoriesView_NothingToPasteMessage;
 
@@ -3072,6 +3153,12 @@ public class UIText extends NLS {
 	/** */
 	public static String UIUtils_StartTypingForPreviousValuesMessage;
 
+	/** */
+	public static String UnmergedBranchDialog_Message;
+
+	/** */
+	public static String UnmergedBranchDialog_Title;
+
 	/** */
 	public static String Untrack_untrack;
 
@@ -3265,6 +3352,9 @@ public class UIText extends NLS {
 	/** */
 	public static String CommitFileDiffViewer_SelectOneCommitMessage;
 
+	/** */
+	public static String CommitFileDiffViewer_ShowAnnotationsMenuLabel;
+
 	/** */
 	public static String CommitGraphTable_CommitId;
 
@@ -3274,6 +3364,12 @@ public class UIText extends NLS {
 	/** */
 	public static String CommitGraphTable_CompareWithEachOtherInTreeMenuLabel;
 
+	/** */
+	public static String CommitGraphTable_DeleteBranchAction;
+
+	/** */
+	public static String CommitGraphTable_messageColumn;
+
 	/** */
 	public static String CommitGraphTable_OpenCommitLabel;
 
@@ -3457,6 +3553,9 @@ public class UIText extends NLS {
 	/** */
 	public static String RefUpdateElement_CommitRangeDecoration;
 
+	/** */
+	public static String RefUpdateElement_statusRejectedNonFastForward;
+
 	/** */
 	public static String RefUpdateElement_UrisDecoration;
 
@@ -3496,6 +3595,24 @@ public class UIText extends NLS {
 	/** */
 	public static String RenameBranchDialog_WindowTitle;
 
+	/** */
+	public static String RevertFailureDialog_Message;
+
+	/** */
+	public static String RevertFailureDialog_MessageNoFiles;
+
+	/** */
+	public static String RevertFailureDialog_ReasonChangesInIndex;
+
+	/** */
+	public static String RevertFailureDialog_ReasonChangesInWorkingDirectory;
+
+	/** */
+	public static String RevertFailureDialog_ReasonDeleteFailure;
+
+	/** */
+	public static String RevertFailureDialog_Title;
+
 	/** */
 	public static String RevertHandler_AlreadyRevertedMessae;
 
@@ -3649,6 +3766,9 @@ public class UIText extends NLS {
 	/** */
 	public static String SimpleConfigurePushDialog_CopyRefSpecButton;
 
+	/** */
+	public static String SimpleConfigurePushDialog_DefaultPushNoRefspec;
+
 	/** */
 	public static String SimpleConfigurePushDialog_DeletePushUriButton;
 
@@ -3697,9 +3817,6 @@ public class UIText extends NLS {
 	/** */
 	public static String SimpleConfigurePushDialog_PasteRefSpecButton;
 
-	/** */
-	public static String SimpleConfigurePushDialog_PushAllBranchesMessage;
-
 	/** */
 	public static String SimpleConfigurePushDialog_PushUrisLabel;
 
@@ -4009,6 +4126,9 @@ public class UIText extends NLS {
 	/** */
 	public static String StagingView_UnstageItemMenuLabel;
 
+	/** */
+	public static String StagingView_StageItemMenuLabel;
+
 	/** */
 	public static String SynchronizeWithMenu_custom;
 
@@ -4042,6 +4162,12 @@ public class UIText extends NLS {
 	/** */
 	public static String GitModelSynchonize_fetchGitDataJobName;
 
+	/** */
+	public static String FetchChangeFromGerritCommand_noRepositorySelectedTitle;
+
+	/** */
+	public static String FetchChangeFromGerritCommand_noRepositorySelectedMessage;
+
 	static {
 		initializeMessages(BUNDLE_NAME, UIText.class);
 	}
