@@ -600,9 +600,13 @@ public class CommitMessageComponent {
 			String text = commitText.getText();
 			int changeIdOffset = findOffsetOfChangeIdLine(text);
 			if (changeIdOffset > 0) {
+				String cleanedText;
 				int endOfChangeId = findNextEOL(changeIdOffset, text);
-				String cleanedText = text.substring(0, changeIdOffset)
-						+ text.substring(endOfChangeId);
+				if (endOfChangeId == -1)
+					cleanedText = text.substring(0, changeIdOffset);
+				else
+					cleanedText = text.substring(0, changeIdOffset)
+							+ text.substring(endOfChangeId);
 				commitText.setText(cleanedText);
 			}
 		}
