@@ -1,6 +1,7 @@
 /*******************************************************************************
  * Copyright (C) 2010, Benjamin Muskalla <bmuskalla@eclipsesource.com>
  * Copyright (C) 2011, Matthias Sohn <matthias.sohn@sap.com>
+ * Copyright (C) 2011, IBM Corporation
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -19,6 +20,7 @@ import java.util.List;
 import java.util.Map;
 
 import org.eclipse.core.runtime.IAdaptable;
+import org.eclipse.egit.core.internal.Utils;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
@@ -33,10 +35,12 @@ import org.eclipse.jface.action.SubMenuManager;
 import org.eclipse.jface.commands.ActionHandler;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.text.Document;
+import org.eclipse.jface.text.ITextListener;
 import org.eclipse.jface.text.ITextOperationTarget;
 import org.eclipse.jface.text.ITextViewer;
 import org.eclipse.jface.text.MarginPainter;
 import org.eclipse.jface.text.Position;
+import org.eclipse.jface.text.TextEvent;
 import org.eclipse.jface.text.contentassist.ICompletionProposal;
 import org.eclipse.jface.text.contentassist.IContentAssistant;
 import org.eclipse.jface.text.quickassist.IQuickAssistInvocationContext;
@@ -188,8 +192,6 @@ public class SpellcheckableMessageArea extends Composite {
 		final SourceViewerDecorationSupport support = configureAnnotationPreferences();
 		final IHandlerActivation handlerActivation = installQuickFixActionHandler();
 
-		configureContextMenu();
-
 		Document document = new Document(initialText);
 
 		sourceViewer.configure(new TextSourceViewerConfiguration(EditorsUI
@@ -199,11 +201,6 @@ public class SpellcheckableMessageArea extends Composite {
 				return getHyperlinkTargets();
 			}
 
-			@Override
-			public int getHyperlinkStateMask(ISourceViewer viewer) {
-				return SWT.NONE;
-			}
-
 			@Override
 			public IReconciler getReconciler(ISourceViewer viewer) {
 				if (!isEditable(viewer))
@@ -233,6 +230,8 @@ public class SpellcheckableMessageArea extends Composite {
 		});
 		sourceViewer.setDocument(document, annotationModel);
 
+		configureContextMenu();
+
 		getTextWidget().addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent disposeEvent) {
 				support.uninstall();
@@ -297,11 +296,23 @@ public class SpellcheckableMessageArea extends Composite {
 		selectAllAction.setText(UIText.SpellCheckingMessageArea_selectAll);
 		selectAllAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_SELECT_ALL);
 
+		final TextViewerAction undoAction = new TextViewerAction(sourceViewer,
+				ITextOperationTarget.UNDO);
+		undoAction.setText(UIText.SpellcheckableMessageArea_undo);
+		undoAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_UNDO);
+
+		final TextViewerAction redoAction = new TextViewerAction(sourceViewer,
+				ITextOperationTarget.REDO);
+		redoAction.setText(UIText.SpellcheckableMessageArea_redo);
+		redoAction.setActionDefinitionId(IWorkbenchCommandConstants.EDIT_REDO);
+
 		MenuManager contextMenu = new MenuManager();
 		contextMenu.add(cutAction);
 		contextMenu.add(copyAction);
 		contextMenu.add(pasteAction);
 		contextMenu.add(selectAllAction);
+		contextMenu.add(undoAction);
+		contextMenu.add(redoAction);
 		contextMenu.add(new Separator());
 
 		if(isEditable(sourceViewer)) {
@@ -323,6 +334,8 @@ public class SpellcheckableMessageArea extends Composite {
 			private IHandlerActivation copyHandlerActivation;
 			private IHandlerActivation pasteHandlerActivation;
 			private IHandlerActivation selectAllHandlerActivation;
+			private IHandlerActivation undoHandlerActivation;
+			private IHandlerActivation redoHandlerActivation;
 
 			public void focusGained(FocusEvent e) {
 				cutAction.update();
@@ -332,11 +345,16 @@ public class SpellcheckableMessageArea extends Composite {
 	            this.copyHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_COPY, new ActionHandler(copyAction), new ActiveShellExpression(getParent().getShell()));
 	            this.pasteHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_PASTE, new ActionHandler(pasteAction), new ActiveShellExpression(getParent().getShell()));
 	            this.selectAllHandlerActivation = service.activateHandler(IWorkbenchCommandConstants.EDIT_SELECT_ALL, new ActionHandler(selectAllAction), new ActiveShellExpression(getParent().getShell()));
+				undoHandlerActivation = service.activateHandler(
+						IWorkbenchCommandConstants.EDIT_UNDO,
+						new ActionHandler(undoAction),
+						new ActiveShellExpression(getParent().getShell()));
+				redoHandlerActivation = service.activateHandler(
+						IWorkbenchCommandConstants.EDIT_REDO,
+						new ActionHandler(redoAction),
+						new ActiveShellExpression(getParent().getShell()));
 			}
 
-			/* (non-Javadoc)
-			 * @see org.eclipse.swt.events.FocusAdapter#focusLost(org.eclipse.swt.events.FocusEvent)
-			 */
 			public void focusLost(FocusEvent e) {
 				IHandlerService service = (IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
 
@@ -355,6 +373,12 @@ public class SpellcheckableMessageArea extends Composite {
 				if (selectAllHandlerActivation != null) {
 					service.deactivateHandler(selectAllHandlerActivation);
 				}
+
+				if (undoHandlerActivation != null)
+					service.deactivateHandler(undoHandlerActivation);
+
+				if (redoHandlerActivation != null)
+					service.deactivateHandler(redoHandlerActivation);
 			}
 
 		});
@@ -367,6 +391,14 @@ public class SpellcheckableMessageArea extends Composite {
 					}
 
         });
+
+		sourceViewer.addTextListener(new ITextListener() {
+
+			public void textChanged(TextEvent event) {
+				undoAction.update();
+				redoAction.update();
+			}
+		});
 	}
 
 	private void addProposals(final SubMenuManager quickFixMenu) {
@@ -515,7 +547,8 @@ public class SpellcheckableMessageArea extends Composite {
 		Action proposalAction = new Action() {
 			public void run() {
 				if (textOperationTarget
-						.canDoOperation(ISourceViewer.CONTENTASSIST_PROPOSALS))
+						.canDoOperation(ISourceViewer.CONTENTASSIST_PROPOSALS)
+						&& getTextWidget().isFocusControl())
 					textOperationTarget
 							.doOperation(ISourceViewer.CONTENTASSIST_PROPOSALS);
 			}
@@ -532,8 +565,7 @@ public class SpellcheckableMessageArea extends Composite {
 	 */
 	public String getCommitMessage() {
 		String text = getText();
-		text = text.replaceAll(getTextWidget().getLineDelimiter(), "\n"); //$NON-NLS-1$
-		return text;
+		return Utils.normalizeLineEndings(text);
 	}
 
 	/**
