@@ -16,13 +16,17 @@ import org.eclipse.compare.ITypedElement;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Path;
+import org.eclipse.egit.core.internal.job.JobUtil;
 import org.eclipse.egit.ui.Activator;
+import org.eclipse.egit.ui.JobFamilies;
+import org.eclipse.egit.ui.UIIcons;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIText;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.CompareUtils;
 import org.eclipse.egit.ui.internal.EgitUiEditorUtils;
 import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput;
+import org.eclipse.egit.ui.internal.blame.BlameOperation;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.MenuManager;
@@ -86,6 +90,8 @@ public class CommitFileDiffViewer extends TableViewer {
 
 	private IAction open;
 
+	private IAction blame;
+
 	private IAction openWorkingTreeVersion;
 
 	private IAction compare;
@@ -194,6 +200,21 @@ public class CommitFileDiffViewer extends TableViewer {
 			}
 		};
 
+		blame = new Action(
+				UIText.CommitFileDiffViewer_ShowAnnotationsMenuLabel,
+				UIIcons.ANNOTATE) {
+			@SuppressWarnings("unchecked")
+			@Override
+			public void run() {
+				final ISelection s = getSelection();
+				if (s.isEmpty() || !(s instanceof IStructuredSelection))
+					return;
+				final IStructuredSelection iss = (IStructuredSelection) s;
+				for (Iterator<FileDiff> it = iss.iterator(); it.hasNext();)
+					showAnnotations(it.next());
+			}
+		};
+
 		openWorkingTreeVersion = new Action(
 				UIText.CommitFileDiffViewer_OpenWorkingTreeVersionInEditorMenuLabel) {
 			@SuppressWarnings("unchecked")
@@ -237,6 +258,7 @@ public class CommitFileDiffViewer extends TableViewer {
 		mgr.add(open);
 		mgr.add(openWorkingTreeVersion);
 		mgr.add(compare);
+		mgr.add(blame);
 
 		mgr.add(new Separator());
 		mgr.add(selectAll = createStandardAction(ActionFactory.SELECT_ALL));
@@ -366,6 +388,38 @@ public class CommitFileDiffViewer extends TableViewer {
 		}
 	}
 
+	private void showAnnotations(FileDiff d) {
+		try {
+			IWorkbenchWindow window = PlatformUI.getWorkbench()
+					.getActiveWorkbenchWindow();
+			IWorkbenchPage page = window.getActivePage();
+			RevCommit commit = d.getChange().equals(ChangeType.DELETE) ? d
+					.getCommit().getParent(0) : d.getCommit();
+			IFileRevision rev = CompareUtils.getFileRevision(d.getPath(),
+					commit, getRepository(),
+					d.getChange().equals(ChangeType.DELETE) ? d.getBlobs()[0]
+							: d.getBlobs()[d.getBlobs().length - 1]);
+			if (rev != null) {
+				BlameOperation op = new BlameOperation(getRepository(),
+						rev.getStorage(new NullProgressMonitor()), d.getPath(),
+						commit, window.getShell(), page);
+				JobUtil.scheduleUserJob(op, UIText.ShowBlameHandler_JobName,
+						JobFamilies.BLAME);
+			} else {
+				String message = NLS.bind(
+						UIText.CommitFileDiffViewer_notContainedInCommit,
+						d.getPath(), d.getCommit().getId().getName());
+				Activator.showError(message, null);
+			}
+		} catch (IOException e) {
+			Activator.logError(UIText.GitHistoryPage_openFailed, e);
+			Activator.showError(UIText.GitHistoryPage_openFailed, null);
+		} catch (CoreException e) {
+			Activator.logError(UIText.GitHistoryPage_openFailed, e);
+			Activator.showError(UIText.GitHistoryPage_openFailed, null);
+		}
+	}
+
 	void showTwoWayFileDiff(final FileDiff d) {
 		final GitCompareFileRevisionEditorInput in;
 
