@@ -69,6 +69,8 @@ import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.Repository;
 import org.eclipse.jgit.revplot.PlotCommit;
+import org.eclipse.jgit.revplot.PlotWalk;
+import org.eclipse.jgit.revwalk.FollowFilter;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.revwalk.RevFlag;
 import org.eclipse.jgit.revwalk.RevObject;
@@ -77,6 +79,7 @@ import org.eclipse.jgit.revwalk.RevTag;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.treewalk.filter.AndTreeFilter;
+import org.eclipse.jgit.treewalk.filter.OrTreeFilter;
 import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
 import org.eclipse.jgit.treewalk.filter.TreeFilter;
 import org.eclipse.osgi.util.NLS;
@@ -225,6 +228,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 		IWorkbenchAction showAllBranchesAction;
 
+		BooleanPrefAction followRenamesAction;
+
 		IWorkbenchAction reuseCompareEditorAction;
 
 		ShowFilterAction showAllRepoVersionsAction;
@@ -256,6 +261,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			createShowNotesAction();
 			createWrapCommentAction();
 			createFillCommentAction();
+			createFollowRenamesAction();
 
 			wrapCommentAction.setEnabled(showCommentAction.isChecked());
 			fillCommentAction.setEnabled(showCommentAction.isChecked());
@@ -360,6 +366,19 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			actionsToDispose.add(showAllBranchesAction);
 		}
 
+		private void createFollowRenamesAction() {
+			followRenamesAction = new BooleanPrefAction(
+					UIPreferences.RESOURCEHISTORY_FOLLOW_RENAMES,
+					UIText.GitHistoryPage_FollowRenames) {
+				@Override
+				void apply(boolean follow) {
+					historyPage.refresh();
+				}
+			};
+			followRenamesAction.apply(followRenamesAction.isChecked());
+			actionsToDispose.add(followRenamesAction);
+		}
+
 		private void createShowCommentAction() {
 			showCommentAction = new BooleanPrefAction(
 					UIPreferences.RESOURCEHISTORY_SHOW_REV_COMMENT,
@@ -433,6 +452,65 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		}
 	}
 
+	/**
+	 * This class defines a couple that associates two pieces of information:
+	 * the file path, and whether it is a regular file (or a directory).
+	 */
+	private static class FilterPath {
+
+		private String path;
+
+		private boolean regularFile;
+
+		public FilterPath(String path, boolean regularFile) {
+			super();
+			this.path = path;
+			this.regularFile = regularFile;
+		}
+
+		/** @return the file path */
+		public String getPath() {
+			return path;
+		}
+
+		/** @return <code>true</code> if the file is a regular file,
+		 * 		and <code>false</code> otherwise (directory, project) */
+		public boolean isRegularFile() {
+			return regularFile;
+		}
+
+		/**
+		 * In {@link FilterPath} class, equality is based on {@link #getPath
+		 * path} equality.
+		 */
+		@Override
+		public boolean equals(Object obj) {
+			if (obj == null || !(obj instanceof FilterPath))
+				return false;
+			FilterPath other = (FilterPath) obj;
+			if (path == null)
+				return other.path == null;
+			return path.equals(other.path);
+		}
+
+		@Override
+		public int hashCode() {
+			if (path != null)
+				return path.hashCode();
+			return super.hashCode();
+		}
+
+		@Override
+		public String toString() {
+			StringBuilder builder = new StringBuilder("Path: "); //$NON-NLS-1$
+			builder.append(getPath());
+			builder.append("regular: "); //$NON-NLS-1$
+			builder.append(isRegularFile());
+
+			return builder.toString();
+		}
+	}
+
 	private static final String POPUP_ID = "org.eclipse.egit.ui.historyPageContributions"; //$NON-NLS-1$
 
 	private static final String DESCRIPTION_PATTERN = "{0} - {1}"; //$NON-NLS-1$
@@ -512,6 +590,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 	private boolean currentShowNotes;
 
+	private boolean currentFollowRenames;
+
 	// react on changes to the relative date preference
 	private final IPropertyChangeListener listener = new IPropertyChangeListener() {
 		public void propertyChange(PropertyChangeEvent event) {
@@ -537,7 +617,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 	 * Note that a change in this list requires that {@link #currentWalk} and
 	 * all of its associated commits.
 	 */
-	private List<String> pathFilters;
+	private List<FilterPath> pathFilters;
 
 	private Runnable refschangedRunnable;
 
@@ -745,9 +825,21 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 					fileViewer.setInput(null);
 					return;
 				}
+
 				final PlotCommit<?> c = (PlotCommit<?>) sel.getFirstElement();
-				commentViewer.setInput(c);
-				fileViewer.setInput(c);
+				final PlotWalk walk = new PlotWalk(input.getRepository());
+				try {
+					final RevCommit unfilteredCommit = walk.parseCommit(c);
+					for (RevCommit parent : unfilteredCommit.getParents())
+						walk.parseBody(parent);
+					commentViewer.setInput(unfilteredCommit);
+					fileViewer.setInput(unfilteredCommit);
+				} catch (IOException e) {
+					commentViewer.setInput(c);
+					fileViewer.setInput(c);
+				} finally {
+					walk.dispose();
+				}
 			}
 		});
 		commentViewer
@@ -806,6 +898,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		viewMenuMgr.add(showSubMenuMgr);
 		showSubMenuMgr.add(actions.showAllBranchesAction);
 		showSubMenuMgr.add(actions.showNotesAction);
+		showSubMenuMgr.add(actions.followRenamesAction);
+		showSubMenuMgr.add(new Separator());
 		showSubMenuMgr.add(actions.findAction);
 		showSubMenuMgr.add(actions.showFilesAction);
 		showSubMenuMgr.add(actions.showCommentAction);
@@ -1377,7 +1471,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 			if (headId == null)
 				return;
 
-			List<String> paths = buildFilterPaths(input.getItems(), input
+			List<FilterPath> paths = buildFilterPaths(input.getItems(), input
 					.getFileList(), db);
 
 			if (forceNewWalk || shouldRedraw(db, headId, paths)) {
@@ -1404,7 +1498,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		}
 	}
 
-	private boolean shouldRedraw(Repository db, AnyObjectId headId, List<String> paths) {
+	private boolean shouldRedraw(Repository db, AnyObjectId headId, List<FilterPath> paths) {
 		boolean pathChanged = pathChanged(pathFilters, paths);
 		boolean headChanged = !headId.equals(currentHeadId);
 		boolean repoChanged = false;
@@ -1418,6 +1512,8 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_NOTES);
 		currentShowNotes = store
 				.getBoolean(UIPreferences.RESOURCEHISTORY_SHOW_NOTES);
+		boolean followRenamesChanged = currentFollowRenames != getFollowRenames();
+		currentFollowRenames = getFollowRenames();
 
 		if (!db.equals(currentRepo)) {
 			repoChanged = true;
@@ -1426,7 +1522,14 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 
 		return pathChanged
 			|| currentWalk == null || headChanged || repoChanged || allBranchesChanged
-			|| showNotesChanged;
+			|| showNotesChanged || followRenamesChanged;
+	}
+
+	/**
+	 * @return whether following renames is currently enabled
+	 */
+	protected boolean getFollowRenames() {
+		return store.getBoolean(UIPreferences.RESOURCEHISTORY_FOLLOW_RENAMES);
 	}
 
 	private AnyObjectId resolveHead(Repository db, boolean acceptNull) {
@@ -1447,12 +1550,12 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		return headId;
 	}
 
-	private ArrayList<String> buildFilterPaths(final IResource[] inResources,
+	private ArrayList<FilterPath> buildFilterPaths(final IResource[] inResources,
 			final File[] inFiles, final Repository db)
 			throws IllegalStateException {
-		final ArrayList<String> paths;
+		final ArrayList<FilterPath> paths;
 		if (inResources != null) {
-			paths = new ArrayList<String>(inResources.length);
+			paths = new ArrayList<FilterPath>(inResources.length);
 			for (final IResource r : inResources) {
 				final RepositoryMapping map = RepositoryMapping.getMapping(r);
 				if (map == null)
@@ -1472,34 +1575,37 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 					else
 						path = map.getRepoRelativePath(r.getParent());
 					if (path != null && path.length() > 0)
-						paths.add(path);
+						paths.add(new FilterPath(path, false));
 				} else if (showAllFilter == ShowFilter.SHOWALLPROJECT) {
 					final String path = map.getRepoRelativePath(r.getProject());
 					if (path != null && path.length() > 0)
-						paths.add(path);
+						paths.add(new FilterPath(path, false));
 				} else if (showAllFilter == ShowFilter.SHOWALLREPO) {
 					// nothing
 				} else /* if (showAllFilter == ShowFilter.SHOWALLRESOURCE) */{
 					final String path = map.getRepoRelativePath(r);
 					if (path != null && path.length() > 0)
-						paths.add(path);
+						paths.add(new FilterPath(path, r.getType() == IResource.FILE));
 				}
 			}
 		} else if (inFiles != null) {
 			IPath workdirPath = new Path(db.getWorkTree().getPath());
 			IPath gitDirPath = new Path(db.getDirectory().getPath());
 			int segmentCount = workdirPath.segmentCount();
-			paths = new ArrayList<String>(inFiles.length);
+			paths = new ArrayList<FilterPath>(inFiles.length);
 			for (File file : inFiles) {
 				IPath filePath;
+				boolean isRegularFile;
 				if (showAllFilter == ShowFilter.SHOWALLFOLDER) {
 					filePath = new Path(file.getParentFile().getPath());
+					isRegularFile = false;
 				} else if (showAllFilter == ShowFilter.SHOWALLPROJECT
 						|| showAllFilter == ShowFilter.SHOWALLREPO) {
 					// we don't know of projects here -> treat as SHOWALLREPO
 					continue;
 				} else /* if (showAllFilter == ShowFilter.SHOWALLRESOURCE) */{
 					filePath = new Path(file.getPath());
+					isRegularFile = file.isFile();
 				}
 
 				if (gitDirPath.isPrefixOf(filePath)) {
@@ -1513,16 +1619,16 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 				IPath pathToAdd = filePath.removeFirstSegments(segmentCount)
 						.setDevice(null);
 				if (!pathToAdd.isEmpty()) {
-					paths.add(pathToAdd.toString());
+					paths.add(new FilterPath(pathToAdd.toString(), isRegularFile));
 				}
 			}
 		} else {
-			paths = new ArrayList<String>(0);
+			paths = new ArrayList<FilterPath>(0);
 		}
 		return paths;
 	}
 
-	private boolean pathChanged(final List<String> o, final List<String> n) {
+	private boolean pathChanged(final List<FilterPath> o, final List<FilterPath> n) {
 		if (o == null)
 			return !n.isEmpty();
 		return !o.equals(n);
@@ -1578,7 +1684,7 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		commentViewer.refresh();
 	}
 
-	private TreeWalk setupFileViewer(Repository db, List<String> paths) {
+	private TreeWalk setupFileViewer(Repository db, List<FilterPath> paths) {
 		final TreeWalk fileWalker = createFileWalker(db, paths);
 		fileViewer.setTreeWalk(db, fileWalker);
 		fileViewer.refresh();
@@ -1586,23 +1692,69 @@ public class GitHistoryPage extends HistoryPage implements RefsChangedListener {
 		return fileWalker;
 	}
 
-	private TreeWalk createFileWalker(Repository db, List<String> paths) {
+	private TreeWalk createFileWalker(Repository db, List<FilterPath> paths) {
 		final TreeWalk fileWalker = new TreeWalk(db);
 		fileWalker.setRecursive(true);
-		if (paths.size() > 0) {
+		fileWalker.setFilter(TreeFilter.ANY_DIFF);
+		if (store.getBoolean(UIPreferences.RESOURCEHISTORY_FOLLOW_RENAMES)
+				&& !paths.isEmpty()
+				&& allRegularFiles(paths)) {
 			pathFilters = paths;
-			currentWalk.setTreeFilter(AndTreeFilter.create(PathFilterGroup
-					.createFromStrings(paths), TreeFilter.ANY_DIFF));
-			fileWalker.setFilter(currentWalk.getTreeFilter().clone());
 
+			List<String> selectedPaths = new ArrayList<String>(paths.size());
+			for (FilterPath filterPath : paths)
+				selectedPaths.add(filterPath.getPath());
+
+			TreeFilter followFilter = createFollowFilterFor(selectedPaths);
+			currentWalk.setTreeFilter(followFilter);
+		} else if (paths.size() > 0) {
+			pathFilters = paths;
+			List<String> stringPaths = new ArrayList<String>(paths.size());
+			for (FilterPath p : paths)
+				stringPaths.add(p.getPath());
+
+			currentWalk.setTreeFilter(AndTreeFilter.create(PathFilterGroup
+					.createFromStrings(stringPaths), TreeFilter.ANY_DIFF));
 		} else {
 			pathFilters = null;
 			currentWalk.setTreeFilter(TreeFilter.ALL);
-			fileWalker.setFilter(TreeFilter.ANY_DIFF);
 		}
 		return fileWalker;
 	}
 
+	/**
+	 * Creates a filter for the given files, will make sure that renames/copies
+	 * of all files will be followed.
+	 * @param paths the list of files to follow, must not be <code>null</code> or empty
+	 * @return the ORed list of {@link FollowFilter follow filters}
+	 */
+	private TreeFilter createFollowFilterFor(List<String> paths) {
+		if (paths == null || paths.isEmpty())
+			throw new IllegalArgumentException("paths must not be null nor empty"); //$NON-NLS-1$
+
+		List<TreeFilter> followFilters = new ArrayList<TreeFilter>(paths.size());
+		for (String path : paths)
+			followFilters.add(FollowFilter.create(path));
+
+		if (followFilters.size() == 1)
+			return followFilters.get(0);
+
+		return OrTreeFilter.create(followFilters);
+	}
+
+	/**
+	 * @return Returns <code>true</code> if <b>all</b> filterpaths refer to plain files,
+	 * 			or if the list is empty.
+	 * @param paths the paths to check
+	 */
+	private boolean allRegularFiles(List<FilterPath> paths) {
+		for (FilterPath filterPath : paths) {
+			if (!filterPath.isRegularFile())
+				return false;
+		}
+		return true;
+	}
+
 	private void scheduleNewGenerateHistoryJob() {
 		final SWTCommitList list = new SWTCommitList(graph.getControl()
 				.getDisplay());
